@@ -65,7 +65,7 @@ class ETTrainer:
         if self.args['pretrained_path'] is not None:
             self.load_checkpoint(self.args['pretrained_path'],
                                  self.args.get('load_model_state', True),
-                                 self.args.get('load_optimizer_state', True))
+                                 self.args.get('load_optimizer_state', False))
 
         elif self.args['phase'] == 'train':
             _torch.manual_seed(self.args['seed'])
@@ -177,7 +177,7 @@ class ETTrainer:
                     checkpoint['optimizers'][k] = self.optimizer[k].state_dict()
         _torch.save(checkpoint, full_path)
 
-    def init_experiment_cache(self):
+    def init_experiment_cache(self, dataspec=None):
         r"""What scores you want to plot."""
         self.cache['log_header'] = 'Loss,Accuracy'
 
@@ -221,34 +221,17 @@ class ETTrainer:
     def evaluation(self,
                    epoch=1,
                    mode='eval',
-                   dataset: list = None,
-                   save_pred=False,
-                   distributed: bool = False,
-                   use_unpadded_sampler: bool = False) -> dict:
-
+                   dataloaders: list = None,
+                   save_pred=False) -> dict:
         for k in self.nn:
             self.nn[k].eval()
 
         eval_avg, eval_metrics = self.new_averages(), self.new_metrics()
 
-        if dataset is None:
+        if not dataloaders:
             return {'averages': eval_avg, 'metrics': eval_metrics}
 
         info(f'{mode} ...', self.args['verbose'])
-        if not isinstance(dataset, list):
-            dataset = [dataset]
-
-        loaders = []
-        for d in dataset:
-            loaders.append(
-                self.data_handle.get_loader(
-                    handle_key=mode,
-                    shuffle=False, dataset=d,
-                    distributed=distributed,
-                    use_unpadded_sampler=use_unpadded_sampler,
-                    reuse=len(dataset) == 1
-                )
-            )
 
         def _update_scores(_out, _it, _avg, _metrics):
             if _out is None:
@@ -257,7 +240,7 @@ class ETTrainer:
             _metrics.accumulate(_out.get('metrics', _it['metrics']))
 
         with _torch.no_grad():
-            for loader in loaders:
+            for loader in dataloaders:
                 its = []
                 metrics = self.new_metrics()
                 avg = self.new_averages()
@@ -269,11 +252,11 @@ class ETTrainer:
                         if self.args['load_sparse']:
                             its.append(it)
                         else:
-                            _update_scores(self.save_predictions(dataset, it), it, avg, metrics)
+                            _update_scores(self.save_predictions(loader.dataset, it), it, avg, metrics)
                     else:
                         _update_scores(None, it, avg, metrics)
 
-                    if self.args['verbose'] and len(dataset) <= 1 and lazy_debug(i, add=epoch):
+                    if self.args['verbose'] and len(dataloaders) <= 1 and lazy_debug(i, add=epoch):
                         info(
                             f" Itr:{i}/{len(loader)}, "
                             f"Averages:{it.get('averages').get()}, Metrics:{it.get('metrics').get()}"
@@ -283,7 +266,7 @@ class ETTrainer:
                     its = self._reduce_iteration(its)
                     _update_scores(self.save_predictions(loader.dataset, its), its, avg, metrics)
 
-                if self.args['verbose'] and len(dataset) > 1:
+                if self.args['verbose'] and len(dataloaders) > 1:
                     info(f" {mode}, {avg.get()}, {metrics.get()}")
 
                 eval_metrics.accumulate(metrics)
@@ -295,9 +278,6 @@ class ETTrainer:
         return {'averages': eval_avg, 'metrics': eval_metrics}
 
     def _reduce_iteration(self, its) -> dict:
-        if len(its) == 1:
-            return its[0]
-
         reduced = {}.fromkeys(its[0].keys(), None)
         for key in reduced:
             if isinstance(its[0][key], _base_metrics.ETAverages):
@@ -417,18 +397,12 @@ class ETTrainer:
                 f"Not best: {val_check['score']}, {self.cache['best_val_score']} in ep: {self.cache['best_val_epoch']}",
                 self.args['verbose'])
 
-    def validation(self, epoch, dataset) -> dict:
-        return self.evaluation(epoch=epoch, mode='validation',
-                               dataset=dataset,
-                               distributed=self.args['use_ddp'],
-                               use_unpadded_sampler=True)
-
     def _global_debug(self, running_averages, running_metrics, **kw):
         """Update running accumulators."""
         running_averages.accumulate(kw.get('averages'))
         running_metrics.accumulate(kw.get('metrics'))
 
-        """Reset iteration accumulator"""
+        """ Reset iteration accumulator """
         N = kw['num_iters']
         i, e = kw['i'], kw['epoch']
 
@@ -444,6 +418,24 @@ class ETTrainer:
 
             running_averages.reset(), running_metrics.reset()
 
+    def inference(self, mode='test', save_predictions=True, datasets: list = None, distributed=False):
+        if not isinstance(datasets, list):
+            datasets = [datasets]
+
+        loaders = []
+        for d in [_d for _d in datasets if _d]:
+            loaders.append(
+                self.data_handle.get_loader(
+                    handle_key=mode, shuffle=False, dataset=d, distributed=distributed
+                )
+            )
+
+        return self.evaluation(
+            mode=mode,
+            dataloaders=[_l for _l in loaders if _l],
+            save_pred=save_predictions
+        )
+
     def train(self, train_dataset, validation_dataset) -> None:
         info('Training ...', self.args['verbose'])
 
@@ -454,6 +446,17 @@ class ETTrainer:
             distributed=self.args['use_ddp']
         )
 
+        val_loader = self.data_handle.get_loader(
+            handle_key='validation',
+            shuffle=False,
+            dataset=validation_dataset,
+            distributed=self.args['use_ddp'],
+            use_unpadded_sampler=True
+        )
+
+        if val_loader is not None and not isinstance(val_loader, list):
+            val_loader = [val_loader]
+
         for ep in range(1, self.args['epochs'] + 1):
             for k in self.nn:
                 self.nn[k].train()
@@ -495,8 +498,8 @@ class ETTrainer:
             epoch_out = {'epoch': ep, 'training': reduced_epoch}
 
             """Validation step"""
-            if validation_dataset is not None:
-                val_out = self.validation(ep, validation_dataset)
+            if val_loader is not None:
+                val_out = self.evaluation(ep, mode='validation', dataloaders=val_loader, save_pred=False)
                 epoch_out['validation'] = self.reduce_scores([val_out], distributed=self.args['use_ddp'])
 
             self._on_epoch_end(**epoch_out)
